#ifndef PC_CODERS3D_H
#define PC_CODERS3D_H

#include <valarray>
#include <cstddef>
#include <istream>
#include <ostream>
#include <vector>
#include "arith.h"

class coder
{
public:
    virtual double encode(compressor &c, const std::valarray<int> &d) = 0;
    virtual std::valarray<int> decode(decompressor &d) = 0;
    virtual ~coder() {}
    virtual double bits() const { return 0.0; }
    virtual double entropy() const { return 0.0; }
    virtual std::size_t use() const { return 1; }
    virtual void dump(std::ostream &out) const = 0;
    virtual void write_header(std::ostream &out) const = 0;
};

class axial_coder : public coder
{
private:
    context axes[2];
    int max_x, max_y;

public:
    axial_coder(int max_x, int max_y);
    axial_coder(std::istream &in);
    virtual double encode(compressor &c, const std::valarray<int> &d);
    virtual std::valarray<int> decode(decompressor &d);
    virtual ~axial_coder() {}
    virtual double bits() const;
    virtual double entropy() const;
    virtual std::size_t use() const;
    virtual void dump(std::ostream &out) const;
    virtual void write_header(std::ostream &out) const;
};

class single_coder : public coder
{
private:
    context freq;
    int max_xy;

public:
    single_coder(int max_x, int max_y);
    single_coder(std::istream &in);
    virtual double encode(compressor &c, const std::valarray<int> &d);
    virtual std::valarray<int> decode(decompressor &d);
    virtual ~single_coder() {}
    virtual double bits() const;
    virtual double entropy() const;
    virtual std::size_t use() const;
    virtual void dump(std::ostream &out) const;
    virtual void write_header(std::ostream &out) const;
};

struct radial_address
{
    short x, y;
};

class radial_coder : public coder
{
private:
    int max_length;
    context rfreq;
    std::vector<int> sizes;
    std::vector<std::vector<int> > index;
    std::vector<std::vector<radial_address> > rindex;
    double total_bits, radial_entropy;

    void initialise(bool compressing);

public:
    radial_coder(int max_length);
    radial_coder(std::istream &in);
    virtual double encode(compressor &c, const std::valarray<int> &d);
    virtual std::valarray<int> decode(decompressor &d);
    virtual double bits() const;
    virtual double entropy() const;
    virtual std::size_t use() const;
    virtual void dump(std::ostream &out) const;
    virtual ~radial_coder() {}
    virtual void write_header(std::ostream &out) const;
};

#endif /* !PC_CODERS3D_H */
