#include <iostream>
#include <valarray>

using namespace std;

template<class T, class S>
static T dot(const valarray<T> &x, const valarray<S> &y)
{
    return x[0] * y[0] + x[1] * y[1] + x[2] * y[2];
}

template<class T>
static inline double length(const valarray<T> &x)
{
    return sqrt((double) x[0] * (double) x[0]
                + (double) x[1] * (double) x[1]
                + (double) x[2] * (double) x[2]);
}

template<class T>
static inline int int_length(const valarray<T> &x)
{
    return (int) round(length(x));
}

static void qresize(valarray<int> &v, int l)
{
    double s = length(v);
    double scale = l / s;
    if (s)
        for (int i = 0; i < 3; i++)
            v[i] = (int) round(v[i] * scale);
}

static void qnormalise(valarray<int> &v)
{
    qresize(v, 0x1000);
}

static valarray<int> rotate_normal(valarray<int> oldf,
                                   valarray<int> newf,
                                   valarray<int> oldu)
{
    int64_t quat[4];
    int64_t qtmp[4];
    int64_t ql2;
    valarray<int> ans(3);
    valarray<int> mid(3);

    // if (is_zero(oldf) || is_zero(newf)) return oldu;
    qnormalise(oldf);
    qnormalise(newf);
    qnormalise(oldu);
    mid = oldf + newf;
    quat[0] = oldf[1] * mid[2] - oldf[2] * mid[1];
    quat[1] = oldf[2] * mid[0] - oldf[0] * mid[2];
    quat[2] = oldf[0] * mid[1] - oldf[1] * mid[0];
    quat[3] = oldf[0] * mid[0] + oldf[1] * newf[1] + oldf[2] * newf[2];
    ql2 = quat[0] * quat[0] + quat[1] * quat[1] + quat[2] * quat[2];

    qtmp[0] = quat[1] * oldu[2] - quat[2] * oldu[1] + quat[3] * oldu[0];
    qtmp[1] = quat[2] * oldu[0] - quat[0] * oldu[2] + quat[3] * oldu[1];
    qtmp[2] = quat[0] * oldu[1] - quat[1] * oldu[0] + quat[3] * oldu[2];
    qtmp[3] = -(quat[0] * oldu[0] + quat[1] * oldu[1] + quat[2] * oldu[2]);

    ans[0] = (qtmp[2] * quat[1] - qtmp[1] * quat[2] - qtmp[3] * quat[0] + quat[3] * qtmp[0]) / ql2;
    ans[1] = (qtmp[0] * quat[2] - qtmp[2] * quat[0] - qtmp[3] * quat[1] + quat[3] * qtmp[1]) / ql2;
    ans[2] = (qtmp[1] * quat[0] - qtmp[0] * quat[1] - qtmp[3] * quat[2] + quat[3] * qtmp[2]) / ql2;
    return ans;
}

int main()
{
    valarray<int> oldf(0, 3), newf(0, 3), oldu(0, 3);

    oldf[0] = 1;
    newf[0] = 1;
    newf[1] = 1;
    oldu[1] = 1;
    valarray<int> newu = rotate_normal(oldf, newf, oldu);
    cout << newu[0] << " " << newu[1] << " " << newu[2] << "\n";
    return 0;
}
